@@ -7,7 +7,7 @@
 #include <glib-object.h>
 
 G_BEGIN_DECLS
-/* enumerations from "/usr/include/gsettings-desktop-schemas/gdesktop-enums.h" */
+/* enumerations from "/home/rmatos/Source/git.gnome.org/install/include/gsettings-desktop-schemas/gdesktop-enums.h" */
 GType g_desktop_proxy_mode_get_type (void) G_GNUC_CONST;
 #define G_DESKTOP_TYPE_DESKTOP_PROXY_MODE (g_desktop_proxy_mode_get_type())
 GType g_desktop_toolbar_style_get_type (void) G_GNUC_CONST;
@@ -28,6 +28,10 @@ GType g_desktop_screensaver_mode_get_type (void) G_GNUC_CONST;
 #define G_DESKTOP_TYPE_DESKTOP_SCREENSAVER_MODE (g_desktop_screensaver_mode_get_type())
 GType g_desktop_magnifier_mouse_tracking_mode_get_type (void) G_GNUC_CONST;
 #define G_DESKTOP_TYPE_DESKTOP_MAGNIFIER_MOUSE_TRACKING_MODE (g_desktop_magnifier_mouse_tracking_mode_get_type())
+GType g_desktop_magnifier_focus_tracking_mode_get_type (void) G_GNUC_CONST;
+#define G_DESKTOP_TYPE_DESKTOP_MAGNIFIER_FOCUS_TRACKING_MODE (g_desktop_magnifier_focus_tracking_mode_get_type())
+GType g_desktop_magnifier_caret_tracking_mode_get_type (void) G_GNUC_CONST;
+#define G_DESKTOP_TYPE_DESKTOP_MAGNIFIER_CARET_TRACKING_MODE (g_desktop_magnifier_caret_tracking_mode_get_type())
 GType g_desktop_magnifier_screen_position_get_type (void) G_GNUC_CONST;
 #define G_DESKTOP_TYPE_DESKTOP_MAGNIFIER_SCREEN_POSITION (g_desktop_magnifier_screen_position_get_type())
 GType g_desktop_titlebar_action_get_type (void) G_GNUC_CONST;
