@@ -58,9 +58,14 @@ enum {
 
 #define W(x) (GtkWidget*) gtk_builder_get_object (priv->builder, x)
 
+#define DATETIME_PERMISSION "org.gnome.controlcenter.datetime.configure"
+
 #define CLOCK_SCHEMA "org.gnome.desktop.interface"
 #define CLOCK_FORMAT_KEY "clock-format"
 
+#define DATETIME_SCHEMA "org.gnome.desktop.datetime"
+#define AUTO_TIMEZONE_KEY "automatic-timezone"
+
 struct _CcDateTimePanelPrivate
 {
   GtkBuilder *builder;
@@ -72,7 +77,8 @@ struct _CcDateTimePanelPrivate
 
   GDateTime *date;
 
-  GSettings *settings;
+  GSettings *clock_settings;
+  GSettings *datetime_settings;
   GDesktopClockFormat clock_format;
   gboolean ampm_available;
   GtkWidget *am_label;
@@ -131,7 +137,7 @@ cc_date_time_panel_dispose (GObject *object)
   g_clear_object (&priv->clock_tracker);
   g_clear_object (&priv->dtm);
   g_clear_object (&priv->permission);
-  g_clear_object (&priv->settings);
+  g_clear_object (&priv->clock_settings);
 
   g_clear_pointer (&priv->date, g_date_time_unref);
 
@@ -183,7 +189,7 @@ change_clock_settings (GObject         *gobject,
   GDesktopClockFormat value;
   const char *active_id;
 
-  g_signal_handlers_block_by_func (priv->settings, clock_settings_changed_cb,
+  g_signal_handlers_block_by_func (priv->clock_settings, clock_settings_changed_cb,
                                    panel);
 
   active_id = gtk_combo_box_get_active_id (GTK_COMBO_BOX (W ("format_combobox")));
@@ -192,12 +198,12 @@ change_clock_settings (GObject         *gobject,
   else
     value = G_DESKTOP_CLOCK_FORMAT_12H;
 
-  g_settings_set_enum (priv->settings, CLOCK_FORMAT_KEY, value);
+  g_settings_set_enum (priv->clock_settings, CLOCK_FORMAT_KEY, value);
   priv->clock_format = value;
 
   update_time (panel);
 
-  g_signal_handlers_unblock_by_func (priv->settings, clock_settings_changed_cb,
+  g_signal_handlers_unblock_by_func (priv->clock_settings, clock_settings_changed_cb,
                                      panel);
 }
 
@@ -539,8 +545,8 @@ update_timezone (CcDateTimePanel *self)
   g_free (label);
 
   /* Translators: UTC here means the Coordinated Universal Time.
-   * %z will be replaced by the offset from UTC e.g. UTC+0200 */
-  utc_label = g_date_time_format (priv->date, _("UTC%z"));
+   * %:::z will be replaced by the offset from UTC e.g. UTC+02 */
+  utc_label = g_date_time_format (priv->date, _("UTC%:::z"));
 
   if (use_ampm)
     {
@@ -599,10 +605,7 @@ get_initial_timezone (CcDateTimePanel *self)
 {
   const gchar *timezone;
 
-  if (self->priv->dtm)
-    timezone = timedate1_get_timezone (self->priv->dtm);
-  else
-    timezone = NULL;
+  timezone = timedate1_get_timezone (self->priv->dtm);
 
   if (timezone == NULL ||
       !cc_timezone_map_set_timezone (CC_TIMEZONE_MAP (self->priv->map), timezone))
@@ -638,19 +641,6 @@ load_regions_model (GtkListStore *cities)
   tz_db_free (db);
 }
 
-static void
-update_widget_state_for_ntp (CcDateTimePanel *panel,
-                             gboolean         using_ntp)
-{
-  CcDateTimePanelPrivate *priv = panel->priv;
-  gboolean allowed;
-
-  /* need to check polkit before revealing to user */
-  allowed = (! priv->permission || g_permission_get_allowed (priv->permission));
-
-  gtk_widget_set_sensitive (W("datetime-button"), !using_ntp && allowed);
-}
-
 static void
 day_changed (GtkWidget       *widget,
              CcDateTimePanel *panel)
@@ -744,7 +734,6 @@ change_ntp (GObject         *gobject,
             GParamSpec      *pspec,
             CcDateTimePanel *self)
 {
-  update_widget_state_for_ntp (self, gtk_switch_get_active (GTK_SWITCH (gobject)));
   queue_set_ntp (self);
 }
 
@@ -754,15 +743,17 @@ on_permission_changed (GPermission *permission,
                        gpointer     data)
 {
   CcDateTimePanelPrivate *priv = CC_DATE_TIME_PANEL (data)->priv;
-  gboolean allowed, using_ntp;
+  gboolean allowed, auto_timezone, using_ntp;
 
-  allowed = g_permission_get_allowed (permission);
+  allowed = (priv->permission == NULL || g_permission_get_allowed (priv->permission));
   using_ntp = gtk_switch_get_active (GTK_SWITCH (W("network_time_switch")));
+  auto_timezone = gtk_switch_get_active (GTK_SWITCH (W("auto_timezone_switch")));
 
   /* All the widgets but the lock button and the 24h setting */
   gtk_widget_set_sensitive (W("auto-datetime-row"), allowed);
-  gtk_widget_set_sensitive (W("timezone-button"), allowed);
-  update_widget_state_for_ntp (data, using_ntp);
+  gtk_widget_set_sensitive (W("auto-timezone-row"), allowed);
+  gtk_widget_set_sensitive (W("datetime-button"), allowed && !using_ntp);
+  gtk_widget_set_sensitive (W("timezone-button"), allowed && !auto_timezone);
 
   /* Hide the subdialogs if we no longer have permissions */
   if (!allowed)
@@ -772,32 +763,13 @@ on_permission_changed (GPermission *permission,
     }
 }
 
-static void
-update_ntp_switch_from_system (CcDateTimePanel *self)
-{
-  CcDateTimePanelPrivate *priv = self->priv;
-  gboolean using_ntp;
-  GtkWidget *switch_widget;
-
-  using_ntp = timedate1_get_ntp (self->priv->dtm);
-
-  switch_widget = W("network_time_switch");
-  g_signal_handlers_block_by_func (switch_widget, change_ntp, self);
-  gtk_switch_set_active (GTK_SWITCH (switch_widget), using_ntp);
-  update_widget_state_for_ntp (self, using_ntp);
-  g_signal_handlers_unblock_by_func (switch_widget, change_ntp, self);
-}
-
 static void
 on_can_ntp_changed (CcDateTimePanel *self)
 {
   CcDateTimePanelPrivate *priv = self->priv;
-  GtkWidget *switch_widget;
-  gboolean sensitive = TRUE;
+  gboolean ntp_available = TRUE;
   GVariant *value;
 
-  switch_widget = W("network_time_switch");
-
   /* We need to access this directly so that we can default to TRUE if
    * it is not set.
    */
@@ -805,17 +777,12 @@ on_can_ntp_changed (CcDateTimePanel *self)
   if (value)
     {
       if (g_variant_is_of_type (value, G_VARIANT_TYPE_BOOLEAN))
-        sensitive = g_variant_get_boolean (value);
+        ntp_available = g_variant_get_boolean (value);
       g_variant_unref (value);
     }
 
-  gtk_widget_set_sensitive (switch_widget, sensitive);
-}
-
-static void
-on_ntp_changed (CcDateTimePanel *self)
-{
-  update_ntp_switch_from_system (self);
+  gtk_widget_set_visible (W ("auto-datetime-row"),
+                          ntp_available);
 }
 
 static void
@@ -893,6 +860,36 @@ run_dialog (CcDateTimePanel *self,
   gtk_dialog_run (GTK_DIALOG (dialog));
 }
 
+static gboolean
+switch_to_row_transform_func (GBinding        *binding,
+                              const GValue    *source_value,
+                              GValue          *target_value,
+                              CcDateTimePanel *self)
+{
+  CcDateTimePanelPrivate *priv = self->priv;
+  gboolean active;
+  gboolean allowed;
+
+  active = g_value_get_boolean (source_value);
+  allowed = (priv->permission == NULL || g_permission_get_allowed (priv->permission));
+
+  g_value_set_boolean (target_value, !active && allowed);
+
+  return TRUE;
+}
+
+static void
+bind_switch_to_row (CcDateTimePanel *self,
+                    GtkWidget       *gtkswitch,
+                    GtkWidget       *listrow)
+{
+  g_object_bind_property_full (gtkswitch, "active",
+                               listrow, "sensitive",
+                               G_BINDING_SYNC_CREATE,
+                               (GBindingTransformFunc) switch_to_row_transform_func,
+                               NULL, self, NULL);
+}
+
 static void
 toggle_switch (GtkWidget *sw)
 {
@@ -922,6 +919,10 @@ list_box_row_activated (GtkListBox      *listbox,
     {
       toggle_switch (W ("network_time_switch"));
     }
+  else if (!g_strcmp0 (widget_name, "auto-timezone-row"))
+    {
+      toggle_switch (W ("auto_timezone_switch"));
+    }
   else if ((found = g_strrstr (widget_name, "button")))
     {
       /* replace "button" with "dialog" */
@@ -1196,6 +1197,7 @@ cc_date_time_panel_init (CcDateTimePanel *self)
   if (priv->dtm == NULL) {
         g_warning ("could not get proxy for DateTimeMechanism: %s", error->message);
         g_clear_error (&error);
+        return;
   }
 
   priv->builder = gtk_builder_new ();
@@ -1211,23 +1213,48 @@ cc_date_time_panel_init (CcDateTimePanel *self)
       return;
     }
 
+  /* add the lock button */
+  priv->permission = polkit_permission_new_sync (DATETIME_PERMISSION, NULL, NULL, NULL);
+  if (priv->permission != NULL)
+    {
+      g_signal_connect (priv->permission, "notify",
+                        G_CALLBACK (on_permission_changed), self);
+      on_permission_changed (priv->permission, NULL, self);
+    }
+  else
+    {
+      g_warning ("Your system does not have the '%s' PolicyKit files installed. Please check your installation",
+                 DATETIME_PERMISSION);
+    }
+
   priv->date = g_date_time_new_now_local ();
 
   setup_timezone_dialog (self);
   setup_datetime_dialog (self);
   setup_main_listview (self);
 
-  /* set up network time button */
-  if (priv->dtm != NULL)
-    {
-      update_ntp_switch_from_system (self);
-      on_can_ntp_changed (self);
-    }
+  /* set up network time switch */
+  bind_switch_to_row (self,
+                      W ("network_time_switch"),
+                      W ("datetime-button"));
+  g_object_bind_property (priv->dtm, "ntp",
+                          W ("network_time_switch"), "active",
+                          G_BINDING_SYNC_CREATE);
   g_signal_connect (W("network_time_switch"), "notify::active",
                     G_CALLBACK (change_ntp), self);
 
+  /* Timezone settings */
+  bind_switch_to_row (self,
+                      W ("auto_timezone_switch"),
+                      W ("timezone-button"));
+
+  priv->datetime_settings = g_settings_new (DATETIME_SCHEMA);
+  g_settings_bind (priv->datetime_settings, AUTO_TIMEZONE_KEY,
+                   W ("auto_timezone_switch"), "active",
+                   G_SETTINGS_BIND_DEFAULT);
+
   /* Clock settings */
-  priv->settings = g_settings_new (CLOCK_SCHEMA);
+  priv->clock_settings = g_settings_new (CLOCK_SCHEMA);
 
   ampm = nl_langinfo (AM_STR);
   /* There are no AM/PM indicators for this locale, so
@@ -1249,8 +1276,8 @@ cc_date_time_panel_init (CcDateTimePanel *self)
   priv->clock_tracker = g_object_new (GNOME_TYPE_WALL_CLOCK, NULL);
   g_signal_connect (priv->clock_tracker, "notify::clock", G_CALLBACK (on_clock_changed), self);
 
-  clock_settings_changed_cb (priv->settings, CLOCK_FORMAT_KEY, self);
-  g_signal_connect (priv->settings, "changed::" CLOCK_FORMAT_KEY,
+  clock_settings_changed_cb (priv->clock_settings, CLOCK_FORMAT_KEY, self);
+  g_signal_connect (priv->clock_settings, "changed::" CLOCK_FORMAT_KEY,
                     G_CALLBACK (clock_settings_changed_cb), self);
 
   g_signal_connect (W("format_combobox"), "notify::active-id",
@@ -1278,29 +1305,11 @@ cc_date_time_panel_init (CcDateTimePanel *self)
                     G_CALLBACK (location_changed_cb), self);
 
   /* Watch changes of timedated remote service properties */
-  if (priv->dtm)
-    {
-      g_signal_connect (priv->dtm, "g-properties-changed",
-                        G_CALLBACK (on_timedated_properties_changed), self);
-      g_signal_connect_swapped (priv->dtm, "notify::ntp",
-                                G_CALLBACK (on_ntp_changed), self);
-      g_signal_connect_swapped (priv->dtm, "notify::can-ntp",
-                                G_CALLBACK (on_can_ntp_changed), self);
-      g_signal_connect_swapped (priv->dtm, "notify::timezone",
-                                G_CALLBACK (on_timezone_changed), self);
-    }
+  g_signal_connect (priv->dtm, "g-properties-changed",
+                    G_CALLBACK (on_timedated_properties_changed), self);
+  g_signal_connect_swapped (priv->dtm, "notify::can-ntp",
+                            G_CALLBACK (on_can_ntp_changed), self);
+  g_signal_connect_swapped (priv->dtm, "notify::timezone",
+                            G_CALLBACK (on_timezone_changed), self);
   /* We ignore UTC <--> LocalRTC changes at the moment */
-
-  /* add the lock button */
-  priv->permission = polkit_permission_new_sync ("org.gnome.controlcenter.datetime.configure", NULL, NULL, NULL);
-  if (priv->permission == NULL)
-    {
-      g_warning ("Your system does not have the '%s' PolicyKit files installed. Please check your installation",
-                 "org.gnome.controlcenter.datetime.configure");
-      return;
-    }
-
-  g_signal_connect (priv->permission, "notify",
-                    G_CALLBACK (on_permission_changed), self);
-  on_permission_changed (priv->permission, NULL, self);
 }
