@@ -33,8 +33,9 @@ G_DEFINE_TYPE (CcTimezoneMap, cc_timezone_map, GTK_TYPE_WIDGET)
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), CC_TYPE_TIMEZONE_MAP, CcTimezoneMapPrivate))
 
 #define PIN_HOT_POINT_X 8
-#define PIN_HOT_POINT_Y 14
+#define PIN_HOT_POINT_Y 15
 
+#define DATETIME_RESOURCE_PATH "/org/gnome/control-center/datetime"
 
 typedef struct
 {
@@ -53,6 +54,7 @@ struct _CcTimezoneMapPrivate
 
   GdkPixbuf *background;
   GdkPixbuf *color_map;
+  GdkPixbuf *pin;
 
   guchar *visible_map_pixels;
   gint visible_map_rowstride;
@@ -61,6 +63,8 @@ struct _CcTimezoneMapPrivate
 
   TzDB *tzdb;
   TzLocation *location;
+
+  gchar *bubble_text;
 };
 
 enum
@@ -149,23 +153,14 @@ cc_timezone_map_dispose (GObject *object)
 
   g_clear_object (&priv->orig_background);
   g_clear_object (&priv->orig_background_dim);
-
-  if (priv->orig_color_map)
-    {
-      g_object_unref (priv->orig_color_map);
-      priv->orig_color_map = NULL;
-    }
-
-  if (priv->background)
-    {
-      g_object_unref (priv->background);
-      priv->background = NULL;
-    }
+  g_clear_object (&priv->orig_color_map);
+  g_clear_object (&priv->background);
+  g_clear_object (&priv->pin);
+  g_clear_pointer (&priv->bubble_text, g_free);
 
   if (priv->color_map)
     {
-      g_object_unref (priv->color_map);
-      priv->color_map = NULL;
+      g_clear_object (&priv->color_map);
 
       priv->visible_map_pixels = NULL;
       priv->visible_map_rowstride = 0;
@@ -195,13 +190,15 @@ cc_timezone_map_get_preferred_width (GtkWidget *widget,
                                      gint      *minimum,
                                      gint      *natural)
 {
-  /* choose a minimum size small enough to prevent the window
-   * from growing horizontally
-   */
+  CcTimezoneMapPrivate *priv = CC_TIMEZONE_MAP (widget)->priv;
+  gint size;
+
+  size = gdk_pixbuf_get_width (priv->orig_background);
+
   if (minimum != NULL)
-    *minimum = 300;
+    *minimum = size;
   if (natural != NULL)
-    *natural = 300;
+    *natural = size;
 }
 
 static void
@@ -212,10 +209,8 @@ cc_timezone_map_get_preferred_height (GtkWidget *widget,
   CcTimezoneMapPrivate *priv = CC_TIMEZONE_MAP (widget)->priv;
   gint size;
 
-  /* The + 20 here is a slight tweak to make the map fill the
-   * panel better without causing horizontal growing
-   */
-  size = 300 * gdk_pixbuf_get_height (priv->orig_background) / gdk_pixbuf_get_width (priv->orig_background) + 20;
+  size = gdk_pixbuf_get_height (priv->orig_background);
+
   if (minimum != NULL)
     *minimum = size;
   if (natural != NULL)
@@ -321,13 +316,84 @@ convert_latitude_to_y (gdouble latitude, gdouble map_height)
   return y;
 }
 
+static void
+draw_text_bubble (cairo_t *cr,
+                  GtkWidget *widget,
+                  gdouble pointx,
+                  gdouble pointy)
+{
+  static const double corner_radius = 9.0;
+  static const double margin_top = 12.0;
+  static const double margin_bottom = 12.0;
+  static const double margin_left = 24.0;
+  static const double margin_right = 24.0;
+
+  CcTimezoneMapPrivate *priv = CC_TIMEZONE_MAP (widget)->priv;
+  GtkAllocation alloc;
+  PangoLayout *layout;
+  PangoRectangle text_rect;
+  double x;
+  double y;
+  double width;
+  double height;
+
+  if (!priv->bubble_text)
+    return;
+
+  gtk_widget_get_allocation (widget, &alloc);
+  layout = gtk_widget_create_pango_layout (widget, NULL);
+
+  /* Layout the text */
+  pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
+  pango_layout_set_spacing (layout, 3);
+  pango_layout_set_markup (layout, priv->bubble_text, -1);
+
+  pango_layout_get_pixel_extents (layout, NULL, &text_rect);
+
+  /* Calculate the bubble size based on the text layout size */
+  width = text_rect.width + margin_left + margin_right;
+  height = text_rect.height + margin_top + margin_bottom;
+
+  if (pointx < alloc.width / 2)
+    x = pointx + 25;
+  else
+    x = pointx - width - 25;
+
+  y = pointy - height / 2;
+
+  /* Make sure it fits in the visible area */
+  x = CLAMP (x, 0, alloc.width - width);
+  y = CLAMP (y, 0, alloc.height - height);
+
+  cairo_save (cr);
+  cairo_translate (cr, x, y);
+
+  /* Draw the bubble */
+  cairo_new_sub_path (cr);
+  cairo_arc (cr, width - corner_radius, corner_radius, corner_radius, radians (-90), radians (0));
+  cairo_arc (cr, width - corner_radius, height - corner_radius, corner_radius, radians (0), radians (90));
+  cairo_arc (cr, corner_radius, height - corner_radius, corner_radius, radians (90), radians (180));
+  cairo_arc (cr, corner_radius, corner_radius, corner_radius, radians (180), radians (270));
+  cairo_close_path (cr);
+
+  cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.7);
+  cairo_fill (cr);
+
+  /* And finally draw the text */
+  cairo_set_source_rgb (cr, 1, 1, 1);
+  cairo_move_to (cr, margin_left, margin_top);
+  pango_cairo_show_layout (cr, layout);
+
+  g_object_unref (layout);
+  cairo_restore (cr);
+}
 
 static gboolean
 cc_timezone_map_draw (GtkWidget *widget,
                       cairo_t   *cr)
 {
   CcTimezoneMapPrivate *priv = CC_TIMEZONE_MAP (widget)->priv;
-  GdkPixbuf *hilight, *orig_hilight, *pin;
+  GdkPixbuf *hilight, *orig_hilight;
   GtkAllocation alloc;
   gchar *file;
   GError *err = NULL;
@@ -343,14 +409,14 @@ cc_timezone_map_draw (GtkWidget *widget,
 
   /* paint hilight */
   if (gtk_widget_is_sensitive (widget))
-    fmt = DATADIR "/timezone_%s.png";
+    fmt = DATETIME_RESOURCE_PATH "/timezone_%s.png";
   else
-    fmt = DATADIR "/timezone_%s_dim.png";
+    fmt = DATETIME_RESOURCE_PATH "/timezone_%s_dim.png";
 
   file = g_strdup_printf (fmt,
                           g_ascii_formatd (buf, sizeof (buf),
                                            "%g", priv->selected_offset));
-  orig_hilight = gdk_pixbuf_new_from_file (file, &err);
+  orig_hilight = gdk_pixbuf_new_from_resource (file, &err);
   g_free (file);
   file = NULL;
 
@@ -373,35 +439,25 @@ cc_timezone_map_draw (GtkWidget *widget,
       g_object_unref (orig_hilight);
     }
 
-  /* load pin icon */
-  pin = gdk_pixbuf_new_from_file (DATADIR "/pin.png", &err);
-
-  if (err)
-    {
-      g_warning ("Could not load pin icon: %s", err->message);
-      g_clear_error (&err);
-    }
-
   if (priv->location)
     {
       pointx = convert_longtitude_to_x (priv->location->longitude, alloc.width);
       pointy = convert_latitude_to_y (priv->location->latitude, alloc.height);
 
-      if (pointy > alloc.height)
-        pointy = alloc.height;
+      pointx = CLAMP (floor (pointx), 0, alloc.width);
+      pointy = CLAMP (floor (pointy), 0, alloc.height);
 
-      if (pin)
+      draw_text_bubble (cr, widget, pointx, pointy);
+
+      if (priv->pin)
         {
-          gdk_cairo_set_source_pixbuf (cr, pin, pointx - PIN_HOT_POINT_X, pointy - PIN_HOT_POINT_Y);
+          gdk_cairo_set_source_pixbuf (cr, priv->pin,
+                                       pointx - PIN_HOT_POINT_X,
+                                       pointy - PIN_HOT_POINT_Y);
           cairo_paint (cr);
         }
     }
 
-  if (pin)
-    {
-      g_object_unref (pin);
-    }
-
   return TRUE;
 }
 
@@ -584,8 +640,8 @@ cc_timezone_map_init (CcTimezoneMap *self)
 
   priv = self->priv = TIMEZONE_MAP_PRIVATE (self);
 
-  priv->orig_background = gdk_pixbuf_new_from_file (DATADIR "/bg.png",
-                                                    &err);
+  priv->orig_background = gdk_pixbuf_new_from_resource (DATETIME_RESOURCE_PATH "/bg.png",
+                                                        &err);
 
   if (!priv->orig_background)
     {
@@ -594,8 +650,8 @@ cc_timezone_map_init (CcTimezoneMap *self)
       g_clear_error (&err);
     }
 
-  priv->orig_background_dim = gdk_pixbuf_new_from_file (DATADIR "/bg_dim.png",
-                                                        &err);
+  priv->orig_background_dim = gdk_pixbuf_new_from_resource (DATETIME_RESOURCE_PATH "/bg_dim.png",
+                                                            &err);
 
   if (!priv->orig_background_dim)
     {
@@ -604,8 +660,8 @@ cc_timezone_map_init (CcTimezoneMap *self)
       g_clear_error (&err);
     }
 
-  priv->orig_color_map = gdk_pixbuf_new_from_file (DATADIR "/cc.png",
-                                                   &err);
+  priv->orig_color_map = gdk_pixbuf_new_from_resource (DATETIME_RESOURCE_PATH "/cc.png",
+                                                       &err);
   if (!priv->orig_color_map)
     {
       g_warning ("Could not load background image: %s",
@@ -613,6 +669,15 @@ cc_timezone_map_init (CcTimezoneMap *self)
       g_clear_error (&err);
     }
 
+  priv->pin = gdk_pixbuf_new_from_resource (DATETIME_RESOURCE_PATH "/pin.png",
+                                            &err);
+  if (!priv->pin)
+    {
+      g_warning ("Could not load pin icon: %s",
+                 (err) ? err->message : "Unknown error");
+      g_clear_error (&err);
+    }
+
   priv->tzdb = tz_load_db ();
 
   g_signal_connect (self, "button-press-event", G_CALLBACK (button_press_event),
@@ -659,6 +724,18 @@ cc_timezone_map_set_timezone (CcTimezoneMap *map,
   return ret;
 }
 
+void
+cc_timezone_map_set_bubble_text (CcTimezoneMap *map,
+                                 const gchar   *text)
+{
+  CcTimezoneMapPrivate *priv = TIMEZONE_MAP_PRIVATE (map);
+
+  g_free (priv->bubble_text);
+  priv->bubble_text = g_strdup (text);
+
+  gtk_widget_queue_draw (GTK_WIDGET (map));
+}
+
 TzLocation *
 cc_timezone_map_get_location (CcTimezoneMap *map)
 {
